#!/usr/bin/python

# reads raw seq of same length (like msa) and computes counts of residues at each position in the protein (written to *_dict_pos.txt" file), as well as counts of residues at each pairwise position (written to "*_dict_pair.txt" file). 

import sys, random, re, math

seqfile = sys.argv[1];
f_seq = open(seqfile, 'r');

aa = ["A","R","N","D","C","E","Q","G","H","I","L","K","M","F","P","S","T","W","Y","V"];
aapair = [];
for aa1 in aa:
	for aa2 in aa:
		aapair.append(aa1+aa2);

msa = [];
for seq in f_seq:
	seq = seq.rstrip('\n');
	msa.append(list(seq));

#seqlen = len(msa[0]);
nseq = len(msa);
seqlen = 99;

############ Functions ###############
def initializeDicts():
	pos_dict = [];
	pair_dict = [];
	for i in range(seqlen):
		pos_dict.append( dict.fromkeys(aa, 0) ); # new dictionary for each position
		pair_dict.append([]); # dic for pairs will have list of dicts for all pairs of positions
		for j in range(seqlen):
			if ( j <= i ): # just to keep indices relevant/correct
				pair_dict[i].append( 0 );
			else:	
				pair_dict[i].append( dict.fromkeys(aapair, 0) );
	return pos_dict, pair_dict;
############	
def printDicts( pos_dict, pair_dict ):

	print "Printing dictionaries:";
	for i in range(len(pos_dict)):
		print "pos ", i;
		for j in range(len(pair_dict[i])):
			print i, j, pair_dict[i][j].values();
		print "pos counts:", pos_dict[i].values();
##### converts list into string with elemets separated by a space
def strList( l ):
	s ="";
	for v in l:
		s = s + " " + str(v);
	return s;	
###################################
def Joint(i,j, msa_passed):     #Function returns a dictionary of the ith and jth position in the seq. and number of not-pairs
	dic = dict.fromkeys(aapair, 0);

	for seq in msa_passed:  # Create list of pairs made from ith and jth position
		pair=seq[i]+seq[j]; 
		if pair in aapair:
			dic[pair]=dic[pair]+1; 

	return dic;    # Return a tuple, the first element is the dictionary of pairs, the other is the number of not-pairs
##################################

########### End of functions #################


true_pos_dict, true_pair_dict = initializeDicts();
# get actual counts from msa
# some sequences are not complete (empty blanks at the end)
for seq in msa:
	for i in range(len(seq)):
		aa1 = seq[i];
		if aa1 in aa:
			true_pos_dict[i][aa1] += 1;
			for j in range(i+1,len(seq)):
				aa2 = seq[j];
				if aa2 in aa:  # only consider the true aa
					true_pair_dict[i][j][aa1+aa2] += 1;
#printDicts( true_pos_dict, true_pair_dict );

#print "54", true_pos_dict[54];
#print true_pos_dict[54].values();
#print "54, 82", true_pair_dict[54][82];
#print true_pair_dict[54][82].values();

h_pos = open(str(seqfile)+'_dict_pos.txt', 'w');
h_pair = open(str(seqfile)+'_dict_pair.txt', 'w');

# open files to write the dictionary counts
for p1 in range(seqlen):
	slist = strList( true_pos_dict[p1].values() );
	h_pos.write(str(p1)+" "+slist+"\n");
	for p2 in range(p1+1, seqlen):
		slist_pair = strList( true_pair_dict[p1][p2].values() );
		h_pair.write(str(p1)+" "+str(p2)+" "+slist_pair+"\n");
